@@ -8,6 +8,7 @@
 
 #include "SpectacleWindow.h"
 
+#include "ExportManager.h"
 #include "SpectacleCore.h"
 #include "spectacle_gui_debug.h"
 
@@ -65,6 +66,8 @@ SpectacleWindow::SpectacleWindow(QQmlEngine *engine, QWindow *parent)
             QCoreApplication::quit();
         }
     });
+    connect(this, &SpectacleWindow::xChanged, this, &SpectacleWindow::logicalXChanged);
+    connect(this, &SpectacleWindow::yChanged, this, &SpectacleWindow::logicalYChanged);
 
     // before we do anything, we need to set a window property
     // that skips the close/hide window animation on kwin. this
@@ -101,6 +104,30 @@ SpectacleWindow::~SpectacleWindow()
     s_spectacleWindowInstances.removeOne(this);
 }
 
+qreal SpectacleWindow::logicalX() const
+{
+#ifdef XCB_FOUND
+    if (KWindowSystem::isPlatformX11()) {
+        return x() / devicePixelRatio();
+    } else
+#endif
+    {
+        return x();
+    }
+}
+
+qreal SpectacleWindow::logicalY() const
+{
+#ifdef XCB_FOUND
+    if (KWindowSystem::isPlatformX11()) {
+        return y() / devicePixelRatio();
+    } else
+#endif
+    {
+        return y();
+    }
+}
+
 ExportMenu *SpectacleWindow::exportMenu() const
 {
     return m_exportMenu.get();
@@ -217,6 +244,7 @@ QString SpectacleWindow::titlePresetString(TitlePreset preset, const QString &fi
 
 void SpectacleWindow::deleter(SpectacleWindow *window)
 {
+    s_spectacleWindowInstances.removeOne(window);
     window->deleteLater();
 }
 
@@ -260,42 +288,23 @@ void SpectacleWindow::setSource(const QUrl &source, const QVariantMap &initialPr
 
 void SpectacleWindow::save()
 {
-    const bool quitChecked = Settings::quitAfterSaveCopyExport();
-    // emits ExportManager::forceNotify when quitChecked is true,
-    // which is connected to SpectacleCore::doNotify,
-    // which emits SpectacleCore::allDone,
-    // which is connected to QCoreApplication::quit in Main.cpp
-    ExportManager::instance()->doSave(QUrl(), /* notify */ quitChecked);
-    if (quitChecked) {
-        SpectacleWindow::closeAll();
-    }
+    ExportManager::instance()->exportImage(ExportManager::Save, SpectacleCore::instance()->outputUrl());
 }
 
 void SpectacleWindow::saveAs()
 {
-    const bool quitChecked = Settings::quitAfterSaveCopyExport();
-    if (ExportManager::instance()->doSaveAs(/* notify */ quitChecked) && quitChecked) {
-        SpectacleWindow::closeAll();
-    }
+    ExportManager::instance()->exportImage(ExportManager::SaveAs);
 }
 
 void SpectacleWindow::copyImage()
 {
-    const bool quitChecked = Settings::quitAfterSaveCopyExport();
-    SpectacleCore::instance()->syncExportPixmap();
-    ExportManager::instance()->doCopyToClipboard(/* notify */ quitChecked);
-    if (quitChecked) {
-        SpectacleWindow::closeAll();
-    }
+    SpectacleCore::instance()->syncExportImage();
+    ExportManager::instance()->exportImage(ExportManager::CopyImage);
 }
 
 void SpectacleWindow::copyLocation()
 {
-    const bool quitChecked = Settings::quitAfterSaveCopyExport();
-    ExportManager::instance()->doCopyLocationToClipboard();
-    if (quitChecked) {
-        SpectacleWindow::closeAll();
-    }
+    ExportManager::instance()->exportImage(ExportManager::CopyPath);
 }
 
 void SpectacleWindow::showPrintDialog()
@@ -452,7 +461,7 @@ void SpectacleWindow::openContainingFolder(const QUrl &url)
 void SpectacleWindow::startDrag()
 {
     auto exportManager = ExportManager::instance();
-    if (exportManager->pixmap().isNull()) {
+    if (exportManager->image().isNull()) {
         return;
     }
 
@@ -480,12 +489,13 @@ void SpectacleWindow::startDrag()
         auto iconSize = units.iconSizes()->large();
         dragHandler->setPixmap(QIcon::fromTheme(QStringLiteral("video-x-matroska")).pixmap(iconSize, iconSize));
     } else {
-        QSize size = exportManager->pixmap().size();
+        QSize size = exportManager->image().size();
+        QPixmap pixmap = QPixmap::fromImage(exportManager->image());
         // TODO: use the composed pixmap with annotations instead
         if (size.width() > 256 || size.height() > 256) {
-            dragHandler->setPixmap(exportManager->pixmap().scaled(256, 256, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+            dragHandler->setPixmap(pixmap.scaled(256, 256, Qt::KeepAspectRatio, Qt::SmoothTransformation));
         } else {
-            dragHandler->setPixmap(exportManager->pixmap());
+            dragHandler->setPixmap(pixmap);
         }
     }
     dragHandler->exec(Qt::CopyAction);
