@@ -6,15 +6,17 @@
  */
 
 #include "SpectacleCore.h"
+#include "CanvasImage.h"
 #include "CaptureModeModel.h"
 #include "CommandLineOptions.h"
+#include "ExportManager.h"
 #include "Gui/Annotations/AnnotationViewport.h"
 #include "Gui/CaptureWindow.h"
 #include "Gui/Selection.h"
 #include "Gui/SelectionEditor.h"
-#include "Gui/SpectacleImageProvider.h"
 #include "Gui/SpectacleWindow.h"
 #include "ShortcutActions.h"
+#include "PlasmaVersion.h"
 // generated
 #include "Config.h"
 #include "settings.h"
@@ -110,31 +112,23 @@ SpectacleCore::SpectacleCore(QObject *parent)
 
     // essential connections
     connect(this, &SpectacleCore::errorMessage, this, &SpectacleCore::showErrorMessage);
-    connect(this, &SpectacleCore::grabDone, this, [this](const QPixmap &pixmap){
-        // only clear images because we're transitioning from rectangle capture to image view.
-        m_annotationDocument->clearImages();
-        if (m_startMode != StartMode::Gui) {
-            SpectacleWindow::setVisibilityForAll(QWindow::Hidden);
-        }
-        onScreenshotUpdated(pixmap);
+    connect(this, &SpectacleCore::grabDone, this, [this](const QImage &image,
+                                                         const ExportManager::Actions &actions){
+        deleteWindows();
+        const auto &exportActions = actions == ExportManager::NoActions ? autoExportActions() : actions;
+        onScreenshotUpdated(image, exportActions);
     });
 
-    connect(platform, &Platform::newScreenshotTaken, this, [this](const QPixmap &pixmap){
-        m_annotationDocument->clear();
-        onScreenshotUpdated(pixmap);
+    connect(platform, &Platform::newScreenshotTaken, this, [this](const QImage &image){
+        m_annotationDocument->clearAnnotations();
+        onScreenshotUpdated(image, autoExportActions());
         setVideoMode(false);
     });
-    connect(platform, &Platform::newScreensScreenshotTaken, this, [this](const QVector<ScreenImage> &screenImages) {
+    connect(platform, &Platform::newScreensScreenshotTaken, this, [this](const QVector<CanvasImage> &screenImages) {
         SelectionEditor::instance()->setScreenImages(screenImages);
         m_annotationDocument->clear();
         for (const auto &img : screenImages) {
-            QImage image(img.image);
-            if (KWindowSystem::isPlatformWayland()) {
-                image.setDevicePixelRatio(qreal(image.width()) / img.screen->geometry().width());
-            } else {
-                image.setDevicePixelRatio(qApp->devicePixelRatio());
-            }
-            m_annotationDocument->addImage(image, img.screen->geometry().topLeft());
+            m_annotationDocument->addImage(img);
         }
 
         auto remember = Settings::rememberLastRectangularRegion();
@@ -151,41 +145,52 @@ SpectacleCore::SpectacleCore(QObject *parent)
 
     // set up the export manager
     auto exportManager = ExportManager::instance();
+    auto onImageExported = [this](const ExportManager::Actions &actions, const QUrl &url) {
+        if (Settings::quitAfterSaveCopyExport()) {
+            deleteWindows();
+        }
+
+        if (isGuiNull()) {
+            if (m_cliOptions[CommandLineOptions::NoNotify]) {
+                // if we notify, we Q_EMIT allDone only if the user either dismissed the notification or pressed
+                // the "Open" button, otherwise the app closes before it can react to it.
+                if (actions & ExportManager::CopyImage) {
+                    // Allow some time for clipboard content to transfer if '--nonotify' is used, see Bug #411263
+                    // TODO: Find better solution
+                    QTimer::singleShot(250, this, &SpectacleCore::allDone);
+                } else {
+                    Q_EMIT allDone();
+                }
+            } else {
+                doNotify(actions, url);
+            }
+            return;
+        }
+
+        auto viewerWindow = ViewerWindow::instance();
+        if (!viewerWindow) {
+            return;
+        }
+
+        if (actions & ExportManager::AnySave) {
+            SpectacleWindow::setTitleForAll(SpectacleWindow::Saved, url.fileName());
+            if (actions & ExportManager::CopyImage) {
+                viewerWindow->showSavedAndCopiedMessage(url);
+            } else if (actions & ExportManager::CopyPath) {
+                viewerWindow->showSavedAndLocationCopiedMessage(url);
+            } else {
+                viewerWindow->showSavedScreenshotMessage(url);
+            }
+        } else if (actions & ExportManager::CopyImage) {
+            viewerWindow->showCopiedMessage();
+        }
+    };
+    connect(exportManager, &ExportManager::imageExported, this, onImageExported);
     connect(exportManager, &ExportManager::errorMessage, this, &SpectacleCore::showErrorMessage);
-    connect(exportManager, &ExportManager::forceNotify, this, &SpectacleCore::doNotify);
+
     connect(platform, &Platform::windowTitleChanged, exportManager, &ExportManager::setWindowTitle);
     connect(m_annotationDocument.get(), &AnnotationDocument::repaintNeeded, m_annotationSyncTimer.get(), qOverload<>(&QTimer::start));
-    connect(m_annotationSyncTimer.get(), &QTimer::timeout, this, &SpectacleCore::syncExportPixmap);
-
-    connect(exportManager, &ExportManager::imageSaved, this, [](const QUrl &savedAt){
-        // This behavior has no relation to the setting in the config UI,
-        // but this was behavior was added to solve this feature request:
-        // https://bugs.kde.org/show_bug.cgi?id=357423
-        if (Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyLocation) {
-            qApp->clipboard()->setText(savedAt.toLocalFile());
-        }
-        SpectacleWindow::setTitleForAll(SpectacleWindow::Saved, savedAt.fileName());
-        if (ViewerWindow::instance()) {
-            ViewerWindow::instance()->showSavedScreenshotMessage(savedAt);
-        }
-    });
-    connect(exportManager, &ExportManager::imageCopied, this, [](){
-        if (ViewerWindow::instance()) {
-            ViewerWindow::instance()->showCopiedMessage();
-        }
-    });
-    connect(exportManager, &ExportManager::imageLocationCopied, this, [](const QUrl &savedAt){
-        SpectacleWindow::setTitleForAll(SpectacleWindow::Saved, savedAt.fileName());
-        if (ViewerWindow::instance()) {
-            ViewerWindow::instance()->showSavedAndLocationCopiedMessage(savedAt);
-        }
-    });
-    connect(exportManager, &ExportManager::imageSavedAndCopied, this, [](const QUrl &savedAt){
-        SpectacleWindow::setTitleForAll(SpectacleWindow::Saved, savedAt.fileName());
-        if (ViewerWindow::instance()) {
-            ViewerWindow::instance()->showSavedAndCopiedMessage(savedAt);
-        }
-    });
+    connect(m_annotationSyncTimer.get(), &QTimer::timeout, this, &SpectacleCore::syncExportImage);
 
     // set up shortcuts
     KGlobalAccel::self()->setGlobalShortcut(ShortcutActions::self()->openAction(), Qt::Key_Print);
@@ -275,6 +280,11 @@ void SpectacleCore::setScreenCaptureUrl(const QString &filePath)
     }
 }
 
+QUrl SpectacleCore::outputUrl() const
+{
+    return m_outputUrl.isEmpty() ? m_editExistingUrl : m_outputUrl;
+}
+
 int SpectacleCore::captureTimeRemaining() const
 {
     int totalDuration = m_delayAnimation->totalDuration();
@@ -318,14 +328,14 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
         }
     }
 
-    // No existing screen capture loaded.
-    m_existingLoaded = false;
-
     // Determine start mode
     m_startMode = StartMode::Gui; // Default to Gui
     // Gui is an option that's normally useless since it's the default mode.
-    // Make it override the other modes if explicitly set or using the launchonly option.
-    if (!m_cliOptions[Option::Gui] && !m_cliOptions[Option::LaunchOnly]) {
+    // Make it override the other modes if explicitly set, using the launchonly option,
+    // or editing an existing image. Editing an existing image requires a viewer window.
+    if (!m_cliOptions[Option::Gui]
+        && !m_cliOptions[Option::LaunchOnly]
+        && !m_cliOptions[Option::EditExisting]) {
         // Background gets precidence over DBus
         if (m_cliOptions[Option::Background]) {
             m_startMode = StartMode::Background;
@@ -341,6 +351,12 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
         deleteWindows();
     }
 
+    // reset last region if it should not be remembered across restarts
+    if (!(Settings::rememberLastRectangularRegion() == Settings::EnumRememberLastRectangularRegion::Always)) {
+        // QRect defaults to {0,0,-1,-1} while QRectF defaults to {0,0,0,0}.
+        Settings::setCropRegion({0, 0, 0, 0});
+    }
+
     /* The logic for setting options for each start mode:
      *
      * - Gui/DBus: Prioritise command line options and default to saved settings.
@@ -365,18 +381,12 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
         onClick = m_cliOptions[Option::OnClick];
         includeDecorations = !m_cliOptions[Option::NoDecoration];
         includePointer = m_cliOptions[Option::Pointer];
-        setSaveCopyImageCopyPath(m_cliOptions[Option::Output],
-                                 m_cliOptions[Option::CopyImage],
-                                 m_cliOptions[Option::CopyPath]);
     } else {
         transientOnly = Settings::transientOnly() || m_cliOptions[Option::TransientOnly];
         onClick = Settings::captureOnClick() || m_cliOptions[Option::OnClick];
         includeDecorations = Settings::includeDecorations()
                             && !m_cliOptions[Option::NoDecoration];
         includePointer = Settings::includePointer() || m_cliOptions[Option::Pointer];
-        setSaveCopyImageCopyPath(m_cliOptions[Option::Output] || Settings::autoSaveImage(),
-                                 m_cliOptions[Option::CopyImage] || Settings::clipboardGroup() == Settings::PostScreenshotCopyImage,
-                                 m_cliOptions[Option::CopyPath] || Settings::clipboardGroup() == Settings::PostScreenshotCopyLocation);
     }
 
     int delayMsec = 0; // default to 0 if cli value parse fails
@@ -390,13 +400,34 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
         }
     }
 
-    m_editExisting = m_cliOptions[Option::EditExisting];
-    if (m_editExisting) {
-        QString existingFileName = parser.value(CommandLineOptions::self()->editExisting);
-        if (!(existingFileName.isEmpty() || existingFileName.isNull())) {
-            setScreenCaptureUrl(existingFileName);
-            m_saveToOutput = true;
+    if (m_cliOptions[Option::EditExisting]) {
+        auto input = parser.value(CommandLineOptions::self()->editExisting);
+        m_editExistingUrl = QUrl::fromUserInput(input, QDir::currentPath(), QUrl::AssumeLocalFile);
+        // QFileInfo::exists() only works with local files.
+        auto existingLocalFile = m_editExistingUrl.toLocalFile();
+        if (QFileInfo::exists(existingLocalFile)) {
+            m_annotationDocument->clearAnnotations();
+            // If editing an existing image, open the annotation editor.
+            // This QImage constructor only works with local files or Qt resource file names.
+            onScreenshotUpdated(QImage(existingLocalFile));
+            return;
+        } else {
+            m_cliOptions[Option::EditExisting] = false;
+            m_editExistingUrl.clear();
         }
+    } else {
+        m_editExistingUrl.clear();
+    }
+
+    if (m_cliOptions[Option::Output]) {
+        m_outputUrl = QUrl::fromUserInput(parser.value(CommandLineOptions::self()->output),
+                                          QDir::currentPath(), QUrl::AssumeLocalFile);
+        if (!m_outputUrl.isValid()) {
+            m_cliOptions[Option::Output] = false;
+            m_outputUrl.clear();
+        }
+    } else {
+        m_outputUrl.clear();
     }
 
     // Determine grab mode
@@ -416,38 +447,18 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
     } else if (Settings::launchAction() == Settings::UseLastUsedCapturemode) {
         grabMode = toGrabMode(CaptureMode(Settings::captureMode()), transientOnly);
     }
-    ExportManager::instance()->setWindowTitle({});
 
-    // reset last region if it should not be remembered across restarts
-    if (!(Settings::rememberLastRectangularRegion() == Settings::EnumRememberLastRectangularRegion::Always)) {
-        // QRect defaults to {0,0,-1,-1} while QRectF defaults to {0,0,0,0}.
-        Settings::setCropRegion({0, 0, 0, 0});
-    }
 
     switch (m_startMode) {
     case StartMode::DBus:
-        m_notify = !m_cliOptions[Option::NoNotify];
         break;
-
-    case StartMode::Background: {
-        m_notify = !m_cliOptions[Option::NoNotify];
-
-        if (m_saveToOutput) {
-            QString lFileName = parser.value(CommandLineOptions::self()->output);
-            if (!(lFileName.isEmpty() || lFileName.isNull())) {
-                setScreenCaptureUrl(lFileName);
-            }
-        }
-
+    case StartMode::Background:
         takeNewScreenshot(grabMode, delayMsec, includePointer, includeDecorations);
-    } break;
-
+        break;
     case StartMode::Gui:
-        m_notify = Settings::quitAfterSaveCopyExport() && !m_cliOptions[Option::NoNotify];
         if (isGuiNull()) {
             if ((m_cliOptions[Option::LaunchOnly]
-                    || Settings::launchAction() == Settings::DoNotTakeScreenshot)
-                && !m_editExisting
+                || Settings::launchAction() == Settings::DoNotTakeScreenshot)
             ) {
                 initViewerWindow(ViewerWindow::Dialog);
                 ViewerWindow::instance()->setVisible(true);
@@ -497,6 +508,16 @@ void SpectacleCore::activate(const QStringList &arguments, const QString &workin
 
 void SpectacleCore::takeNewScreenshot(Platform::GrabMode grabMode, int timeout, bool includePointer, bool includeDecorations)
 {
+    if (m_cliOptions[CommandLineOptions::EditExisting]) {
+        // Clear when a new screenshot is taken to avoid overwriting
+        // the existing file with a completely unrelated image.
+        m_editExistingUrl.clear();
+        m_cliOptions[CommandLineOptions::EditExisting] = false;
+    }
+
+    // Clear the window title that can be used in file names.
+    ExportManager::instance()->setWindowTitle({});
+
     m_delayAnimation->stop();
 
     m_lastGrabMode = grabMode;
@@ -513,12 +534,14 @@ void SpectacleCore::takeNewScreenshot(Platform::GrabMode grabMode, int timeout,
 
     const bool noDelay = timeout == 0;
 
-    // when compositing is enabled, we need to give it enough time for the window
-    // to disappear and all the effects are complete before we take the shot. there's
-    // no way of knowing how long the disappearing effects take, but as per default
-    // settings (and unless the user has set an extremely slow effect), 200
-    // milliseconds is a good amount of wait time.
-    timeout = qMax(timeout, KX11Extras::compositingActive() ? 200 : 50);
+    if (PlasmaVersion::get() < PlasmaVersion::check(5, 27, 5)) {
+        // when compositing is enabled, we need to give it enough time for the window
+        // to disappear and all the effects are complete before we take the shot. there's
+        // no way of knowing how long the disappearing effects take, but as per default
+        // settings (and unless the user has set an extremely slow effect), 200
+        // milliseconds is a good amount of wait time.
+        timeout = qMax(timeout, KX11Extras::compositingActive() ? 200 : 50);
+    }
 
     if (noDelay) {
         SpectacleWindow::setVisibilityForAll(QWindow::Hidden);
@@ -555,99 +578,49 @@ void SpectacleCore::cancelScreenshot()
     }
 }
 
-void SpectacleCore::showErrorMessage(const QString &theErrString)
+void SpectacleCore::showErrorMessage(const QString &message)
 {
-    qCDebug(SPECTACLE_CORE_LOG) << "ERROR: " << theErrString;
+    qCDebug(SPECTACLE_CORE_LOG) << "ERROR: " << message;
 
     if (m_startMode == StartMode::Gui) {
-        KMessageBox::error(nullptr, theErrString);
+        KMessageBox::error(nullptr, message);
     }
 }
 
-void SpectacleCore::onScreenshotUpdated(const QPixmap &thePixmap)
+void SpectacleCore::onScreenshotUpdated(const QImage &image, const ExportManager::Actions &actions)
 {
     using Option = CommandLineOptions::Option;
-    QPixmap existingPixmap;
-    const QPixmap &pixmapUsed = (m_editExisting && !m_existingLoaded) ? existingPixmap : thePixmap;
-    if (m_editExisting && !m_existingLoaded) {
-        existingPixmap.load(m_screenCaptureUrl.toLocalFile());
-    }
+    using Action = ExportManager::Action;
+
+    m_annotationDocument->clearImages();
 
     auto exportManager = ExportManager::instance();
-    exportManager->setPixmap(pixmapUsed);
-    m_annotationDocument->addImage(pixmapUsed.toImage(), QPointF(0, 0));
-    exportManager->updatePixmapTimestamp();
+    exportManager->setImage(image);
+    m_annotationDocument->addImage(image);
+    exportManager->updateTimestamp();
 
-    switch (m_startMode) {
-    case StartMode::Background:
-    case StartMode::DBus: {
-        syncExportPixmap();
-        if (m_saveToOutput) {
-            QUrl lSavePath = (m_startMode == StartMode::Background && m_screenCaptureUrl.isValid() && m_screenCaptureUrl.isLocalFile()) ? m_screenCaptureUrl : QUrl();
-            exportManager->doSave(lSavePath, m_notify);
-        }
-        if (m_copyImageToClipboard) {
-            exportManager->doCopyToClipboard(m_notify);
-        } else if (m_copyLocationToClipboard) {
-            exportManager->doCopyLocationToClipboard(m_notify);
-        }
-
-        // if we don't have a Gui already opened, Q_EMIT allDone
-        if (isGuiNull()) {
-            // if we notify, we Q_EMIT allDone only if the user either dismissed the notification or pressed
-            // the "Open" button, otherwise the app closes before it can react to it.
-            if (!m_notify && m_copyImageToClipboard) {
-                // Allow some time for clipboard content to transfer if '--nonotify' is used, see Bug #411263
-                // TODO: Find better solution
-                QTimer::singleShot(250, this, &SpectacleCore::allDone);
-            } else if (!m_notify) {
-                Q_EMIT allDone();
-            }
-        }
-    } break;
-    case StartMode::Gui:
-        // These can change in GUI mode, so set them again
-        m_notify = Settings::quitAfterSaveCopyExport()
-                && !m_cliOptions[CommandLineOptions::NoNotify];
-        setSaveCopyImageCopyPath(m_cliOptions[Option::Output] || Settings::autoSaveImage(),
-                                 m_cliOptions[Option::CopyImage] || Settings::clipboardGroup() == Settings::PostScreenshotCopyImage,
-                                 m_cliOptions[Option::CopyPath] || Settings::clipboardGroup() == Settings::PostScreenshotCopyLocation);
-
-        if (pixmapUsed.isNull()) {
+    if (m_startMode == StartMode::Gui) {
+        if (image.isNull()) {
             initViewerWindow(ViewerWindow::Dialog);
             ViewerWindow::instance()->setVisible(true);
             return;
         }
-        if (!m_editExisting) {
-            setScreenCaptureUrl(QUrl(QStringLiteral("image://spectacle/%1").arg(pixmapUsed.cacheKey())));
-        }
         initViewerWindow(ViewerWindow::Image);
-        if (m_editExisting) {
+        if (m_cliOptions[Option::EditExisting]) {
             ViewerWindow::instance()->setAnnotating(true);
         }
         ViewerWindow::instance()->setVisible(true);
-        auto titlePreset = !pixmapUsed.isNull() ? SpectacleWindow::Unsaved : SpectacleWindow::Saved;
+        auto titlePreset = !image.isNull() ? SpectacleWindow::Unsaved : SpectacleWindow::Saved;
         SpectacleWindow::setTitleForAll(titlePreset);
-
-        if (m_saveToOutput && m_copyImageToClipboard) {
-            syncExportPixmap();
-            exportManager->doSaveAndCopy();
-        } else if (m_saveToOutput) {
-            exportManager->doSave();
-        } else if (m_copyImageToClipboard) {
-            syncExportPixmap();
-            exportManager->doCopyToClipboard(false);
-        }
-        // This is a separate block since we don't do anything special for
-        // saving and copying the location, unlike saving and copying the image.
-        if (m_copyLocationToClipboard) {
-            exportManager->doCopyLocationToClipboard(false);
-        }
     }
 
-    if (m_editExisting && !m_existingLoaded) {
-        Settings::setLastSaveLocation(m_screenCaptureUrl);
-        m_existingLoaded = true;
+    syncExportImage();
+    if (actions & Action::Save && m_cliOptions[Option::Output]) {
+        exportManager->exportImage(actions, m_outputUrl);
+    } else if (actions & Action::Save && m_cliOptions[Option::EditExisting]) {
+        exportManager->exportImage(actions, m_editExistingUrl);
+    } else {
+        exportManager->exportImage(actions, outputUrl());
     }
 }
 
@@ -673,7 +646,7 @@ void SpectacleCore::onScreenshotFailed()
 
 static QVector<KNotification *> notifications;
 
-void SpectacleCore::doNotify(const QUrl &theSavedAt)
+void SpectacleCore::doNotify(const ExportManager::Actions &actions, const QUrl &saveUrl)
 {
     // ensure program stays alive until the notification finishes.
     if (!m_eventLoopLocker) {
@@ -690,35 +663,41 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
     notification->setTitle(captureModeLabel.toString());
 
     // a speaking message is prettier than a URL, special case for copy image/location to clipboard and the default pictures location
-    const QString &lSavePath = theSavedAt.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path();
+    const QString &saveDirPath = saveUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path();
+    const QString &saveFileName = saveUrl.fileName();
 
-    if (m_copyImageToClipboard && theSavedAt.fileName().isEmpty()) {
+    using Action = ExportManager::Action;
+    if (actions & Action::AnySave && !saveFileName.isEmpty()) {
+        if (actions & Action::CopyPath) {
+            notification->setText(i18n("A screenshot was saved as '%1' to '%2' and the file path of the screenshot has been saved to your clipboard.",
+                                       saveFileName, saveDirPath));
+        } else if (saveDirPath == QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)) {
+            notification->setText(i18nc("Placeholder is filename",
+                                        "A screenshot was saved as '%1' to your Pictures folder.",
+                                        saveFileName));
+        } else {
+            notification->setText(i18n("A screenshot was saved as '%1' to '%2'.",
+                                       saveFileName, saveDirPath));
+        }
+    } else if (actions & Action::CopyImage) {
         notification->setText(i18n("A screenshot was saved to your clipboard."));
-    } else if (m_copyLocationToClipboard && !theSavedAt.fileName().isEmpty()) {
-        notification->setText(i18n("A screenshot was saved as '%1' to '%2' and the file path of the screenshot has been saved to your clipboard.",
-                              theSavedAt.fileName(),
-                              lSavePath));
-    } else if (lSavePath == QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)) {
-        notification->setText(i18nc("Placeholder is filename", "A screenshot was saved as '%1' to your Pictures folder.", theSavedAt.fileName()));
-    } else if (!theSavedAt.fileName().isEmpty()) {
-        notification->setText(i18n("A screenshot was saved as '%1' to '%2'.", theSavedAt.fileName(), lSavePath));
     }
 
-    if (!theSavedAt.isEmpty()) {
-        notification->setUrls({theSavedAt});
+    if (!saveUrl.isEmpty()) {
+        notification->setUrls({saveUrl});
         notification->setDefaultAction(i18nc("Open the screenshot we just saved", "Open"));
-        connect(notification, &KNotification::defaultActivated, this, [theSavedAt]() {
-            auto job = new KIO::OpenUrlJob(theSavedAt);
+        connect(notification, &KNotification::defaultActivated, this, [saveUrl]() {
+            auto job = new KIO::OpenUrlJob(saveUrl);
             job->start();
         });
         notification->setActions({i18n("Annotate")});
-        connect(notification, &KNotification::action1Activated, this, [theSavedAt]() {
+        connect(notification, &KNotification::action1Activated, this, [saveUrl]() {
             QProcess newInstance;
             newInstance.setProgram(QCoreApplication::applicationFilePath());
             newInstance.setArguments({
                 CommandLineOptions::toArgument(CommandLineOptions::self()->newInstance),
                 CommandLineOptions::toArgument(CommandLineOptions::self()->editExisting),
-                theSavedAt.toLocalFile()
+                saveUrl.toLocalFile()
             });
             newInstance.startDetached();
         });
@@ -737,18 +716,29 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
     notification->sendEvent();
 }
 
-// In background and dbus mode, ensure that either save or copy image is enabled.
-void SpectacleCore::setSaveCopyImageCopyPath(bool save, bool copyImage, bool copyPath)
+ExportManager::Actions SpectacleCore::autoExportActions() const
 {
-    if (m_startMode == StartMode::Gui) {
-        m_saveToOutput = save;
-        m_copyImageToClipboard = copyImage;
-        m_copyLocationToClipboard = save && copyPath;
-    } else {
-        m_saveToOutput = save || !copyImage;
-        m_copyImageToClipboard = !m_saveToOutput || copyImage;
-        m_copyLocationToClipboard = save && copyPath;
+    using Action = ExportManager::Action;
+    using Option = CommandLineOptions::Option;
+    bool save = m_startMode != StartMode::Gui && m_cliOptions[Option::Output];
+    bool copyImage = m_cliOptions[Option::CopyImage];
+    bool copyPath = m_cliOptions[Option::CopyPath];
+    ExportManager::Actions actions;
+    if (m_startMode != StartMode::Background) {
+        save |= Settings::autoSaveImage();
+        copyImage |= Settings::clipboardGroup() == Settings::PostScreenshotCopyImage;
+        copyPath |= Settings::clipboardGroup() == Settings::PostScreenshotCopyLocation;
     }
+    if (m_startMode == StartMode::Gui) {
+        actions.setFlag(Action::Save, save);
+        actions.setFlag(Action::CopyImage, copyImage);
+    } else {
+        // In background and dbus mode, ensure that either save or copy image is enabled.
+        actions.setFlag(Action::Save, save || !copyImage);
+        actions.setFlag(Action::CopyImage, !actions.testFlag(Action::Save) || copyImage);
+    }
+    actions.setFlag(Action::CopyPath, actions.testFlag(Action::Save) && copyPath);
+    return actions;
 }
 
 Platform::GrabMode SpectacleCore::toGrabMode(CaptureModeModel::CaptureMode captureMode, bool transientOnly) const
@@ -813,23 +803,21 @@ void SpectacleCore::initGuiNoScreenshot()
     ViewerWindow::instance()->setVisible(true);
 }
 
-void SpectacleCore::syncExportPixmap()
+void SpectacleCore::syncExportImage()
 {
     qreal maxDpr = 0.0;
-    for (auto &img : m_annotationDocument->baseImages()) {
-        maxDpr = qMax(maxDpr, img.devicePixelRatio());
+    for (auto &img : m_annotationDocument->canvasImages()) {
+        maxDpr = qMax(maxDpr, img.image.devicePixelRatio());
     }
     QRectF imageRect(QPointF(0, 0), m_annotationDocument->canvasSize() * maxDpr);
     const auto &image = m_annotationDocument->renderToImage(imageRect, maxDpr);
-    ExportManager::instance()->setPixmap(QPixmap::fromImage(image));
+    ExportManager::instance()->setImage(image);
 }
 
 QQmlEngine *SpectacleCore::getQmlEngine()
 {
     if (m_engine == nullptr) {
         m_engine = std::make_unique<QQmlEngine>(this);
-        m_engine->addImageProvider(QStringLiteral("spectacle"),
-                                   new SpectacleImageProvider(QQmlImageProviderBase::Pixmap));
         m_engine->rootContext()->setContextObject(new KLocalizedContext(m_engine.get()));
 
         qmlRegisterSingletonInstance(QML_URI_PRIVATE, 1, 0, "SpectacleCore", this);
